<?php

namespace EventPortal\Domain\Field\Models;

use EventPortal\Concerns\HasSchemalessAttributes;
use EventPortal\Domain\Field\Builders\FieldBuilder;
use EventPortal\Domain\Field\Enums\FieldType;
use EventPortal\Domain\Response\Models\Response;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Spatie\Enum\Laravel\HasEnums;

class Field extends Model
{
    use HasSchemalessAttributes, HasEnums;

    /** @var array<string> */
    protected $casts = [
        'data' => 'array',
    ];

    /** @var array<string> */
    protected $enums = [
        'type' => FieldType::class,
    ];

    /** @var array<string> */
    protected $guarded = [];

    /**
     * @param \Illuminate\Database\Query\Builder $query
     * @phpcsSuppress SlevomatCodingStandard.TypeHints.ParameterTypeHint.MissingNativeTypeHint
     */
    public function newEloquentBuilder($query): FieldBuilder
    {
        return new FieldBuilder($query);
    }

    public function response(): HasOne
    {
        return $this->hasOne(Response::class);
    }

    public function children(): HasMany
    {
        return $this->hasMany(Field::class, 'section_id');
    }

    public function isChild(): bool
    {
        return $this->section_id !== null;
    }

    public function isParent(): bool
    {
        return $this->section_id === null;
    }
}
